(* Ce module implante le compilateur proprement dit. *)

open Expression
open Buffer

let b =
  Buffer.create 1024

let printf format =
  Printf.bprintf b format

let prelude () = printf "
        .data
nl:     .asciiz \"\\n\"
        .text
        .globl __start
__start:
Loop:
        li   $v0, 5                    # la primitive read_int
        syscall
        beq  $v0, 0, Fin               # si l'entier lu vaut zro, on arrte
        move $a0, $v0                  # sinon on le place dans $a0 et on value l'expression
        jal  eval
        move $a0, $v0                  # puis on affiche le rsultat obtenu
        jal  writeln
        j    Loop                      # et on recommence
writeln:
        li   $v0, 1                    # la primitive print_int
        syscall 
        la   $a0, nl
	li   $v0, 4                    # la primitive print_string
	syscall
        jr   $ra
eval:
"

let postlude () = printf "\t\
        jr $ra
Fin:
	li  $v0, 10	 # Exit
	syscall
"

(* La fonction [compile_expression e] produit un fragment de code assembleur. Ce
   code suppose que la valeur de la variable [x] se trouve dans le registre [$a0].
   Ce code doit calculer la valeur de l'expression [e] et placer cette valeur dans
   le registre [$v0]. *)

let rec compile_expression = function
  | Int i ->
      printf "        li   $v0, %d\n" i
  | X     ->
      printf "        move $v0, $a0\n" 
  | _     ->
      failwith "Pas implment!"

(* La fonction [compile e] produit un programme assembleur complet. Ce
   programme, excut  l'aide de spim, effectue une boucle: il lit une valeur
   entire sur l'entre standard; si cette valeur est nulle, le programme
   termine; si cette valeur est non nulle, alors le programme associe cette
   valeur  la variable [x], puis calcule et affiche la valeur de l'expression
   [e], et recommence. *)

let compile e =
  prelude();
  compile_expression e;
  postlude();
  print_string (Buffer.contents b);
  flush stdout

